@@ -22,8 +22,6 @@ from dataclasses import dataclass, field
 from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Protocol, Tuple, Union
 
-from src.templating.template_context_validator import clear_global_template_cache
-from src.utils.log_phases import PhaseLogger
 from src.string_utils import (
     log_debug_safe,
     log_error_safe,
@@ -31,6 +29,8 @@ from src.string_utils import (
     log_warning_safe,
     safe_format,
 )
+from src.templating.template_context_validator import clear_global_template_cache
+from src.utils.log_phases import PhaseLogger
 
 # Import board functions from the correct module
 from .device_clone.constants import PRODUCTION_DEFAULTS
@@ -100,7 +100,7 @@ def _optional_int(value: Optional[Union[int, str]]) -> Optional[int]:
         return None
 
 
-@dataclass
+@dataclass(slots=True)
 class BuildConfiguration:
     """Configuration for the firmware build process."""
 
@@ -121,7 +121,7 @@ class BuildConfiguration:
     enable_error_injection: bool = False
 
 
-@dataclass
+@dataclass(slots=True)
 class MSIXData:
     """Container for MSI-X capability data."""
 
@@ -131,7 +131,7 @@ class MSIXData:
     config_space_bytes: Optional[bytes] = None
 
 
-@dataclass
+@dataclass(slots=True)
 class DeviceConfiguration:
     """Device configuration extracted from the build process."""
 
@@ -312,7 +312,7 @@ class MSIXManager:
             Returns empty MSIXData on any failure (non-critical operation)
         """
         try:
-            log_info_safe(self.logger, "➤ Preloading MSI-X data before VFIO binding")
+            log_info_safe(self.logger, "Preloading MSI-X data before VFIO binding")
 
             # 1) Prefer host-provided JSON (mounted into container) if available
             #    This preserves MSI-X context when container lacks sysfs/VFIO access.
@@ -331,9 +331,11 @@ class MSIXManager:
                     if msix_info and isinstance(msix_info, dict):
                         log_info_safe(
                             self.logger,
-                            "  • Loaded MSI-X from {path} ({vectors} vectors)",
-                            path=msix_json_path,
-                            vectors=msix_info.get("table_size", 0),
+                            safe_format(
+                                "Loaded MSI-X from {path} ({vectors} vectors)",
+                                path=msix_json_path,
+                                vectors=msix_info.get("table_size", 0),
+                            ),
                             prefix="MSIX",
                         )
                         return MSIXData(
@@ -348,8 +350,10 @@ class MSIXManager:
                 # Non-fatal; fall back to sysfs path
                 log_debug_safe(
                     self.logger,
-                    "MSI-X JSON ingestion skipped: {err}",
-                    err=str(e),
+                    safe_format(
+                        "MSI-X JSON ingestion skipped: {err}",
+                        err=str(e),
+                    ),
                     prefix="MSIX",
                 )
 
@@ -369,8 +373,10 @@ class MSIXManager:
             if msix_info["table_size"] > 0:
                 log_info_safe(
                     self.logger,
-                    "  • Found MSI-X capability: {vectors} vectors",
-                    vectors=msix_info["table_size"],
+                    safe_format(
+                        "Found MSI-X capability: {vectors} vectors",
+                        vectors=msix_info["table_size"],
+                    ),
                     prefix="MSIX",
                 )
                 return MSIXData(
@@ -382,20 +388,19 @@ class MSIXManager:
             else:
                 # No MSI-X capability found -> treat as not preloaded so callers
                 # don't assume hardware MSI-X values are available.
-                log_info_safe(
-                    self.logger, "  • No MSI-X capability found", prefix="MSIX"
-                )
+                log_info_safe(self.logger, "No MSI-X capability found", prefix="MSIX")
                 return MSIXData(preloaded=False, msix_info=None)
 
         except Exception as e:
             log_warning_safe(
-                self.logger, "MSI-X preload failed: {err}", err=str(e), prefix="MSIX"
+                self.logger,
+                safe_format("MSI-X preload failed: {err}", err=str(e)),
+                prefix="MSIX",
             )
             if self.logger.isEnabledFor(logging.DEBUG):
                 log_debug_safe(
                     self.logger,
-                    "MSI-X preload exception details: {err}",
-                    err=str(e),
+                    safe_format("MSI-X preload exception details: {err}", err=str(e)),
                     prefix="MSIX",
                 )
             return MSIXData(preloaded=False)
@@ -411,7 +416,9 @@ class MSIXManager:
         if not self._should_inject(msix_data):
             return
 
-        log_info_safe(self.logger, "  • Using preloaded MSI-X data", prefix="MSIX")
+        log_info_safe(
+            self.logger, safe_format("Using preloaded MSI-X data"), prefix="MSIX"
+        )
 
         # msix_info is guaranteed to be non-None by _should_inject
         if msix_data.msix_info is not None:
@@ -888,7 +895,10 @@ class ConfigurationManager:
         # Validate profile duration
         if args.profile < 0:
             raise ConfigurationError(
-                f"Invalid profile duration: {args.profile}. Must be >= 0"
+                safe_format(
+                    "Invalid profile duration: {profile}. Must be >= 0",
+                    profile=args.profile,
+                )
             )
 
     def _is_valid_bdf(self, bdf: str) -> bool:
@@ -1002,12 +1012,17 @@ class FirmwareBuilder:
             return self.file_manager.list_artifacts()
 
         except PlatformCompatibilityError:
+            # Reraise platform compatibility errors without modification
             raise
         except Exception as e:
-            log_error_safe(self.logger, "Build failed: {err}", err=str(e))
-            if self.logger.isEnabledFor(logging.DEBUG):
-                log_debug_safe(self.logger, "Full traceback while building")
-            raise
+            log_error_safe(
+                self.logger,
+                safe_format("Build failed: {err}", err=str(e)),
+                prefix="BUILD",
+            )
+            raise PCILeechBuildError(
+                safe_format("Build failed: {err}", err=str(e))
+            ) from e
 
     def run_vivado(self) -> None:
         """
@@ -1027,8 +1042,10 @@ class FirmwareBuilder:
             vivado_path = self.config.vivado_path
             log_info_safe(
                 self.logger,
-                "Using user-specified Vivado path: {path}",
-                path=vivado_path,
+                safe_format(
+                    "Using user-specified Vivado path: {path}", path=vivado_path
+                ),
+                prefix="VIVADO",
             )
         else:
             # Auto-detect Vivado installation
@@ -1045,8 +1062,8 @@ class FirmwareBuilder:
             vivado_path = str(vivado_exe_path.parent.parent)
             log_info_safe(
                 self.logger,
-                "Auto-detected Vivado at: {path}",
-                path=vivado_path,
+                safe_format("Auto-detected Vivado at: {path}", path=vivado_path),
+                prefix="VIVADO",
             )
 
         # Create and run VivadoRunner
@@ -1104,20 +1121,29 @@ class FirmwareBuilder:
 
             log_info_safe(
                 self.logger,
-                "Loading donor template from: {path}",
-                path=self.config.donor_template,
+                safe_format(
+                    "Loading donor template from: {path}",
+                    path=self.config.donor_template,
+                ),
+                prefix="BUILD",
             )
             try:
                 template = DonorInfoTemplateGenerator.load_template(
                     self.config.donor_template
                 )
-                log_info_safe(self.logger, "✓ Donor template loaded successfully")
+                log_info_safe(
+                    self.logger, "Donor template loaded successfully", prefix="BUILD"
+                )
                 return template
             except Exception as e:
                 log_error_safe(
-                    self.logger, "Failed to load donor template: {err}", err=str(e)
+                    self.logger,
+                    safe_format("Failed to load donor template: {err}", err=str(e)),
+                    prefix="BUILD",
                 )
-                raise PCILeechBuildError(f"Failed to load donor template: {e}")
+                raise PCILeechBuildError(
+                    safe_format("Failed to load donor template: {err}", err=str(e))
+                ) from e
         return None
 
     def _preload_msix(self) -> MSIXData:
@@ -1187,8 +1213,7 @@ class FirmwareBuilder:
             # Log but do not fail build if hex generation fails
             log_warning_safe(
                 self.logger,
-                "Config space hex generation failed: {err}",
-                err=str(e),
+                safe_format("Config space hex generation failed: {err}", err=str(e)),
                 prefix="BUILD",
             )
 
@@ -1206,16 +1231,17 @@ class FirmwareBuilder:
                 json.dump(audit, f, indent=2)
             log_debug_safe(
                 self.logger,
-                "Template context audit written ({count} keys) → {path}",
-                count=len(keys),
-                path=str(audit_path),
+                safe_format(
+                    "Template context audit written ({count} keys) → {path}",
+                    count=len(keys),
+                    path=str(audit_path),
+                ),
                 prefix="BUILD",
             )
         except Exception as e:
             log_debug_safe(
                 self.logger,
-                "Template context audit skipped: {err}",
-                err=str(e),
+                safe_format("Template context audit skipped: {err}", err=str(e)),
                 prefix="BUILD",
             )
 
@@ -1237,9 +1263,11 @@ class FirmwareBuilder:
         group_id = ensure_device_vfio_binding(self.config.bdf)
         log_warning_safe(
             self.logger,
-            "VFIO binding recheck passed: bdf={bdf} group={group}",
-            bdf=self.config.bdf,
-            group=str(group_id),
+            safe_format(
+                "VFIO binding recheck passed: bdf={bdf} group={group}",
+                bdf=self.config.bdf,
+                group=str(group_id),
+            ),
             prefix="VFIO",
         )
 
@@ -1255,17 +1283,21 @@ class FirmwareBuilder:
 
         log_info_safe(
             self.logger,
-            "  • Wrote {count} SystemVerilog modules: {files}",
-            count=len(sv_files),
-            files=", ".join(sv_files),
+            safe_format(
+                "Wrote {count} SystemVerilog modules: {files}",
+                count=len(sv_files),
+                files=", ".join(sv_files),
+            ),
             prefix="BUILD",
         )
         if special_files:
             log_info_safe(
                 self.logger,
-                "  • Wrote {count} special files: {files}",
-                count=len(special_files),
-                files=", ".join(special_files),
+                safe_format(
+                    "Wrote {count} special files: {files}",
+                    count=len(special_files),
+                    files=", ".join(special_files),
+                ),
                 prefix="BUILD",
             )
 
@@ -1278,7 +1310,7 @@ class FirmwareBuilder:
             self.file_manager.write_json("behavior_profile.json", profile)
             log_info_safe(
                 self.logger,
-                "  • Saved behavior profile → behavior_profile.json",
+                "Saved behavior profile to behavior_profile.json",
                 prefix="BUILD",
             )
 
@@ -1286,12 +1318,27 @@ class FirmwareBuilder:
         """Generate TCL scripts for Vivado."""
         ctx = result["template_context"]
         device_config = ctx["device_config"]
+
+        # Validate board is present and non-empty
+        board = self.config.board
+        if not board or not board.strip():
+            raise ConfigurationError(
+                "Board name is required for TCL generation. "
+                "Use --board to specify a valid board configuration "
+                "(e.g., pcileech_100t484_x1)"
+            )
+
         # Extract optional subsystem IDs
         subsys_vendor_id = _optional_int(device_config.get("subsystem_vendor_id"))
         subsys_device_id = _optional_int(device_config.get("subsystem_device_id"))
 
+        # Extract PCIe link speed and width from template context
+        # These are critical for donor-unique firmware generation
+        pcie_max_link_speed = ctx.get("pcie_max_link_speed")
+        pcie_max_link_width = ctx.get("pcie_max_link_width")
+
         self.tcl.build_all_tcl_scripts(
-            board=self.config.board,
+            board=board,
             device_id=device_config["device_id"],
             class_code=device_config["class_code"],
             revision_id=device_config["revision_id"],
@@ -1300,6 +1347,8 @@ class FirmwareBuilder:
             subsys_device_id=subsys_device_id,
             build_jobs=self.config.vivado_jobs,
             build_timeout=self.config.vivado_timeout,
+            pcie_max_link_speed_code=pcie_max_link_speed,
+            pcie_max_link_width=pcie_max_link_width,
         )
 
         log_info_safe(
@@ -1361,8 +1410,9 @@ class FirmwareBuilder:
             generator.save_template_dict(template, output_path, pretty=True)
             log_info_safe(
                 self.logger,
-                "  • Generated donor info template → {name}",
-                name=output_path.name,
+                safe_format(
+                    "Generated donor info template {name}", name=output_path.name
+                ),
                 prefix="BUILD",
             )
 
@@ -1616,6 +1666,7 @@ def main(argv: Optional[List[str]] = None) -> int:
             log_info_safe(
                 logger,
                 "Build skipped due to platform compatibility (see details above)",
+                prefix="BUILD",
             )
         else:
             # Unexpected errors
@@ -1745,14 +1796,14 @@ def _maybe_emit_issue_report(
         if repro_cmd:
             log_info_safe(
                 logger,
-                "Reproduce with: {cmd}",
-                cmd=repro_cmd,
+                safe_format("Reproduce with: {cmd}", cmd=repro_cmd),
+                prefix="BUILD",
             )
     except Exception as emit_err:  # pragma: no cover - best effort
         log_warning_safe(
             logger,
-            "Issue report generation failed: {err}",
-            err=str(emit_err),
+            safe_format("Issue report generation failed: {err}", err=str(emit_err)),
+            prefix="BUILD",
         )
 
 
