@@ -32,7 +32,7 @@ except ImportError:
     from src.log_config import get_logger, setup_logging
     from src.shell import Shell
 
-from ..string_utils import log_info_safe, log_warning_safe, safe_format
+from ..string_utils import log_error_safe, log_info_safe, log_warning_safe, safe_format
 from .build_constants import (
     DEFAULT_ACTIVE_INTERRUPT_MODE,
     DEFAULT_ACTIVE_INTERRUPT_VECTOR,
@@ -354,6 +354,17 @@ def main(argv: Optional[List[str]] = None):
             donor_template=getattr(args, "donor_template", None),
             enable_error_injection=getattr(args, "enable_error_injection", False),
         )
+
+        # Validate board parameter before container launch to fail fast
+        if not board or not board.strip():
+            log_error_safe(
+                logger,
+                "Board name is required. Use --board to specify a valid board "
+                "configuration (e.g., pcileech_100t484_x1)",
+                prefix="BUILD",
+            )
+            sys.exit(2)
+
         run_build(cfg)
 
     elif args.cmd == "flash":
