@@ -21,13 +21,21 @@ import re
 import shlex
 import statistics
 import subprocess
+import sys
 import threading
 import time
 from dataclasses import asdict, dataclass
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple
 
-
+from src.device_clone.manufacturing_variance import (
+    DeviceClass,
+    ManufacturingVarianceSimulator,
+)
+from src.error_utils import extract_root_cause
+from src.exceptions import PlatformCompatibilityError
+from src.log_config import get_logger
+from src.scripts.kernel_utils import setup_debugfs
 from src.string_utils import (
     log_debug_safe,
     log_error_safe,
@@ -36,16 +44,6 @@ from src.string_utils import (
     safe_format,
 )
 
-from src.log_config import get_logger
-from src.exceptions import PlatformCompatibilityError
-from src.scripts.kernel_utils import setup_debugfs
-
-from src.device_clone.manufacturing_variance import (
-    DeviceClass,
-    ManufacturingVarianceSimulator,
-)
-from src.error_utils import extract_root_cause
-
 
 def is_linux() -> bool:
     """Check if running on Linux."""
@@ -64,7 +62,7 @@ def check_linux_requirement(operation: str) -> None:
         )
 
 
-@dataclass
+@dataclass(slots=True)
 class RegisterAccess:
     """Represents a single register access event."""
 
@@ -76,7 +74,7 @@ class RegisterAccess:
     duration_us: Optional[float] = None
 
 
-@dataclass
+@dataclass(slots=True)
 class TimingPattern:
     """Represents a timing pattern in register accesses."""
 
@@ -88,7 +86,7 @@ class TimingPattern:
     confidence: float
 
 
-@dataclass
+@dataclass(slots=True)
 class BehaviorProfile:
     """Complete behavioral profile of a device."""
 
@@ -323,9 +321,11 @@ class BehaviorProfiler:
         except Exception as e:
             log_warning_safe(
                 self.logger,
-                "Ftrace setup failed (may require root): {error}",
+                safe_format(
+                    "Ftrace setup failed (may require root): {error}",
+                    error=e,
+                ),
                 prefix="PROFILER",
-                error=e,
             )
             # Disable ftrace for this session since it's not working
             self.enable_ftrace = False
@@ -349,9 +349,11 @@ class BehaviorProfiler:
             except Exception as e:
                 log_error_safe(
                     self.logger,
-                    "Monitor worker error: {error}",
+                    safe_format(
+                        "Monitor worker error: {error}",
+                        error=e,
+                    ),
                     prefix="PROFILER",
-                    error=e,
                 )
                 break
 
@@ -384,8 +386,7 @@ class BehaviorProfiler:
             if Path(trace_path).exists():
                 # Non-blocking read of trace events
                 result = subprocess.run(
-                    f"timeout 0.001 cat {trace_path}",
-                    shell=True,
+                    ["timeout", "0.001", "cat", trace_path],
                     capture_output=True,
                     text=True,
                 )
@@ -398,18 +399,22 @@ class BehaviorProfiler:
             # unavailable
             log_debug_safe(
                 self.logger,
-                "Ftrace monitoring unavailable: {error}",
+                safe_format(
+                    "Ftrace monitoring unavailable: {error}",
+                    error=e,
+                ),
                 prefix="PROFILER",
-                error=e,
             )
             # Disable ftrace for future calls to avoid repeated errors
             self.enable_ftrace = False
         except Exception as e:
             log_warning_safe(
                 self.logger,
-                "Ftrace monitoring error: {error}",
+                safe_format(
+                    "Ftrace monitoring error: {error}",
+                    error=e,
+                ),
                 prefix="PROFILER",
-                error=e,
             )
 
     def _monitor_sysfs_accesses(self) -> None:
@@ -439,16 +444,20 @@ class BehaviorProfiler:
             # Expected in some environments
             log_debug_safe(
                 self.logger,
-                "Sysfs monitoring limited: {error}",
+                safe_format(
+                    "Sysfs monitoring limited: {error}",
+                    error=e,
+                ),
                 prefix="PROFILER",
-                error=e,
             )
         except Exception as e:
             log_warning_safe(
                 self.logger,
-                "Sysfs monitoring error: {error}",
+                safe_format(
+                    "Sysfs monitoring error: {error}",
+                    error=e,
+                ),
                 prefix="PROFILER",
-                error=e,
             )
 
     def _monitor_debugfs_registers(self) -> None:
@@ -476,9 +485,11 @@ class BehaviorProfiler:
             except Exception as e:
                 log_warning_safe(
                     self.logger,
-                    "Failed to setup debugfs for register monitoring: {error}",
+                    safe_format(
+                        "Failed to setup debugfs for register monitoring: {error}",
+                        error=e,
+                    ),
                     prefix="PROFILER",
-                    error=e,
                 )
                 self.debugfs_available = False
                 # Continue without debugfs monitoring - this is not critical for basic functionality
@@ -505,18 +516,22 @@ class BehaviorProfiler:
             # Expected when debugfs is not available or accessible
             log_debug_safe(
                 self.logger,
-                "Debugfs monitoring unavailable: {error}",
+                safe_format(
+                    "Debugfs monitoring unavailable: {error}",
+                    error=e,
+                ),
                 prefix="PROFILER",
-                error=e,
             )
             # Disable debugfs for future iterations
             self.debugfs_available = False
         except Exception as e:
             log_warning_safe(
                 self.logger,
-                "Debugfs monitoring error: {error}",
+                safe_format(
+                    "Debugfs monitoring error: {error}",
+                    error=e,
+                ),
                 prefix="PROFILER",
-                error=e,
             )
 
     def _parse_ftrace_output(self, output: str) -> None:
@@ -1489,36 +1504,32 @@ class BehaviorProfiler:
             if "cmd" in access.register.lower() or "ctrl" in access.register.lower()
         ]
 
-        if command_registers:
-            # Calculate command processing characteristics
-            if len(command_registers) > 1:
-                intervals = []
-                for i in range(1, len(command_registers)):
-                    interval = (
-                        command_registers[i].timestamp
-                        - command_registers[i - 1].timestamp
-                    ) * 1000000
-                    intervals.append(interval)
+        if not command_registers or len(command_registers) <= 1:
+            return command_patterns
 
-                if intervals:
-                    command_patterns["avg_command_latency_us"] = sum(intervals) / len(
-                        intervals
-                    )
-                    command_patterns["command_frequency_hz"] = (
-                        1000000 / command_patterns["avg_command_latency_us"]
-                    )
+        # Calculate command processing characteristics
+        intervals = []
+        for i in range(1, len(command_registers)):
+            interval = (
+                command_registers[i].timestamp - command_registers[i - 1].timestamp
+            ) * 1000000
+            intervals.append(interval)
 
-                    # Calculate regularity
-                    if len(intervals) > 1:
-                        import statistics
+        if not intervals:
+            return command_patterns
 
-                        std_dev = statistics.stdev(intervals)
-                        avg_interval = command_patterns["avg_command_latency_us"]
-                        command_patterns["command_regularity"] = (
-                            max(0, 1 - (std_dev / avg_interval))
-                            if avg_interval > 0
-                            else 0
-                        )
+        command_patterns["avg_command_latency_us"] = sum(intervals) / len(intervals)
+        command_patterns["command_frequency_hz"] = (
+            1000000 / command_patterns["avg_command_latency_us"]
+        )
+
+        # Calculate regularity
+        if len(intervals) > 1:
+            std_dev = statistics.stdev(intervals)
+            avg_interval = command_patterns["avg_command_latency_us"]
+            command_patterns["command_regularity"] = (
+                max(0, 1 - (std_dev / avg_interval)) if avg_interval > 0 else 0
+            )
 
         return command_patterns
 
@@ -1807,4 +1818,4 @@ def main():
 
 
 if __name__ == "__main__":
-    exit(main())
+    sys.exit(main())
