@@ -15,17 +15,11 @@ from pathlib import Path
 from typing import Any, Dict, List
 
 from ..__version__ import __version__
-from ..device_clone.constants import PCILEECH_BUILD_SCRIPT, PCILEECH_PROJECT_SCRIPT
-from ..string_utils import (
-    get_project_name,
-    log_debug_safe,
-    log_error_safe,
-    log_info_safe,
-    log_warning_safe,
-    safe_format,
-    safe_print_format,
-    format_kv_table,
-)
+from ..device_clone.constants import (PCILEECH_BUILD_SCRIPT,
+                                      PCILEECH_PROJECT_SCRIPT)
+from ..string_utils import (format_kv_table, get_project_name, log_debug_safe,
+                            log_error_safe, log_info_safe, log_warning_safe,
+                            safe_format, safe_print_format)
 
 logger = logging.getLogger(__name__)
 
