@@ -10,37 +10,13 @@ the new modular implementation internally.
 import logging
 from typing import Dict, List, Optional
 
+from ..string_utils import (log_debug_safe, log_error_safe, log_info_safe,
+                            log_warning_safe, safe_format)
 from .core import CapabilityWalker, ConfigSpace
 from .processor import CapabilityProcessor
 from .rules import RuleEngine
 from .types import CapabilityType, EmulationCategory, PatchInfo, PruningAction
 
-try:
-    from ..string_utils import (
-        log_debug_safe,
-        log_error_safe,
-        log_info_safe,
-        log_warning_safe,
-        safe_format,
-    )
-except ImportError:
-    import os
-    import sys
-    from pathlib import Path
-
-    # Add parent directory to sys.path for direct import if needed
-    current_dir = Path(__file__).parent
-    parent_dir = str(current_dir)
-    if parent_dir not in sys.path:
-        sys.path.insert(0, parent_dir)
-    from string_utils import (
-        log_debug_safe,
-        log_error_safe,
-        log_info_safe,
-        log_warning_safe,
-        safe_format,
-    )
-
 logger = logging.getLogger(__name__)
 
 
@@ -269,7 +245,8 @@ def determine_pruning_actions(
         )
         cap_infos[offset] = cap_info
 
-    from .utils import determine_pruning_actions as utils_determine_pruning_actions
+    from .utils import \
+        determine_pruning_actions as utils_determine_pruning_actions
 
     return utils_determine_pruning_actions(cap_infos, categories)
 
