@@ -11,13 +11,8 @@ binary format.
 import logging
 from typing import Dict, List, Optional, Set, Tuple
 
-from ..string_utils import (
-    log_debug_safe,
-    log_error_safe,
-    log_info_safe,
-    log_warning_safe,
-    safe_format,
-)
+from ..string_utils import (log_debug_safe, log_error_safe, log_info_safe,
+                            log_warning_safe, safe_format)
 from .core import ConfigSpace
 from .types import PatchInfo, PruningAction
 
