@@ -8,11 +8,8 @@ reduce import complexity.
 """
 
 import logging
-
 from dataclasses import dataclass, field
-
 from enum import Enum
-
 from typing import Dict, List, Optional, Set
 
 from src.utils import validation_constants as VC
@@ -22,7 +19,9 @@ try:
     from ..__version__ import __version__ as PCILEECH_FWGEN_VERSION  # type: ignore
 except Exception:  # pragma: no cover - fallback if package structure differs
     try:
-        from src.__version__ import __version__ as PCILEECH_FWGEN_VERSION  # type: ignore
+        from src.__version__ import (
+            __version__ as PCILEECH_FWGEN_VERSION,
+        )  # type: ignore
     except Exception:
         PCILEECH_FWGEN_VERSION = "unknown"
 
@@ -53,6 +52,9 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
+_DEFAULT_TRANSITION_CYCLES = VC.get_power_transition_cycles()
+
+
 class PowerState(Enum):
     """PCIe power states."""
 
@@ -169,10 +171,18 @@ class TransitionCycles:
     """Power state transition cycle counts."""
 
     # Default to centralized validation constants; allow overrides
-    d0_to_d1: int = VC.POWER_TRANSITION_CYCLES.get("d0_to_d1", 100)
-    d1_to_d0: int = VC.POWER_TRANSITION_CYCLES.get("d1_to_d0", 200)
-    d0_to_d3: int = VC.POWER_TRANSITION_CYCLES.get("d0_to_d3", 500)
-    d3_to_d0: int = VC.POWER_TRANSITION_CYCLES.get("d3_to_d0", 1000)
+    d0_to_d1: int = field(
+        default_factory=lambda: _DEFAULT_TRANSITION_CYCLES["d0_to_d1"]
+    )
+    d1_to_d0: int = field(
+        default_factory=lambda: _DEFAULT_TRANSITION_CYCLES["d1_to_d0"]
+    )
+    d0_to_d3: int = field(
+        default_factory=lambda: _DEFAULT_TRANSITION_CYCLES["d0_to_d3"]
+    )
+    d3_to_d0: int = field(
+        default_factory=lambda: _DEFAULT_TRANSITION_CYCLES["d3_to_d0"]
+    )
 
 
 @dataclass
@@ -210,6 +220,7 @@ class AdvancedFeatureConfig:
     enable_assertions: bool = True
     enable_coverage: bool = False
     clock_frequency_mhz: int = 250
+    prefix: str = "ADV_SV_FEATURES"
 
 
 class AdvancedSVFeatureGenerator:
@@ -218,10 +229,11 @@ class AdvancedSVFeatureGenerator:
     def __init__(self, config: AdvancedFeatureConfig):
         self.config = config
         self.renderer = TemplateRenderer()
+        self.prefix = config.prefix
         log_info_safe(
             logger,
             "Initialized AdvancedSVFeatureGenerator with config",
-            prefix="GENERATOR",
+            prefix=self.config.prefix,
         )
 
     def generate_error_handling_module(self) -> str:
@@ -230,11 +242,11 @@ class AdvancedSVFeatureGenerator:
             log_debug_safe(
                 logger,
                 "Error handling disabled, returning empty module",
-                prefix="ERROR_GEN",
+                prefix=self.prefix,
             )
             return ""
 
-        log_info_safe(logger, "Generating error handling module", prefix="ERROR_GEN")
+        log_info_safe(logger, "Generating error handling module", prefix=self.prefix)
 
         try:
             # Import here to avoid circular imports
@@ -279,7 +291,7 @@ class AdvancedSVFeatureGenerator:
                     "Failed to import error handling generator: {error}",
                     error=str(e),
                 ),
-                prefix="ERROR_GEN",
+                prefix=self.prefix,
             )
             return self._generate_fallback_error_module()
         except Exception as e:
@@ -289,7 +301,7 @@ class AdvancedSVFeatureGenerator:
                     "Error generating error handling module: {error}",
                     error=str(e),
                 ),
-                prefix="ERROR_GEN",
+                prefix=self.prefix,
             )
             return self._generate_fallback_error_module()
 
@@ -299,12 +311,12 @@ class AdvancedSVFeatureGenerator:
             log_debug_safe(
                 logger,
                 "Performance monitoring disabled, returning empty module",
-                prefix="PERF_GEN",
+                prefix=self.prefix,
             )
             return ""
 
         log_info_safe(
-            logger, "Generating performance monitoring module", prefix="PERF_GEN"
+            logger, "Generating performance monitoring module", prefix=self.prefix
         )
 
         try:
@@ -329,7 +341,7 @@ class AdvancedSVFeatureGenerator:
                     "Error generating performance monitor module: {error}",
                     error=str(e),
                 ),
-                prefix="PERF_GEN",
+                prefix=self.prefix,
             )
             return self._generate_fallback_performance_module()
 
@@ -339,11 +351,11 @@ class AdvancedSVFeatureGenerator:
             log_debug_safe(
                 logger,
                 "Power management disabled, returning empty module",
-                prefix="POWER_GEN",
+                prefix=self.prefix,
             )
             return ""
 
-        log_info_safe(logger, "Generating power management module", prefix="POWER_GEN")
+        log_info_safe(logger, "Generating power management module", prefix=self.prefix)
 
         try:
             context = {
@@ -367,14 +379,14 @@ class AdvancedSVFeatureGenerator:
                     "Error generating power management module: {error}",
                     error=str(e),
                 ),
-                prefix="POWER_GEN",
+                prefix=self.prefix,
             )
             return self._generate_fallback_power_module()
 
     def _generate_fallback_error_module(self) -> str:
         """Generate a fallback error handling module when template generation fails."""
         log_warning_safe(
-            logger, "Using fallback error handling module", prefix="FALLBACK"
+            logger, "Using fallback error handling module", prefix=self.prefix
         )
 
         context = {"config": self.config.error_handling}
@@ -388,7 +400,7 @@ class AdvancedSVFeatureGenerator:
     def _generate_fallback_performance_module(self) -> str:
         """Generate a fallback performance monitoring module when template generation fails."""
         log_warning_safe(
-            logger, "Using fallback performance monitoring module", prefix="FALLBACK"
+            logger, "Using fallback performance monitoring module", prefix=self.prefix
         )
 
         context = {"config": self.config.performance}
@@ -403,7 +415,7 @@ class AdvancedSVFeatureGenerator:
     def _generate_fallback_power_module(self) -> str:
         """Generate a fallback power management module when template generation fails."""
         log_warning_safe(
-            logger, "Using fallback power management module", prefix="FALLBACK"
+            logger, "Using fallback power management module", prefix=self.prefix
         )
 
         context = {"config": self.config.power_management}
@@ -426,7 +438,7 @@ class AdvancedSVFeatureGenerator:
                     "Generating module template for {module}",
                     module=module_name,
                 ),
-                prefix="TEMPLATE",
+                prefix=self.prefix,
             )
 
             # Try to use Jinja2 template first
@@ -443,7 +455,7 @@ class AdvancedSVFeatureGenerator:
                         "Template {template_name} not found, using fallback generation",
                         template_name=template_name,
                     ),
-                    prefix="TEMPLATE",
+                    prefix=self.prefix,
                 )
                 return self._generate_fallback_module(module_name, context, *components)
 
@@ -454,7 +466,7 @@ class AdvancedSVFeatureGenerator:
                     "Error in template generation: {error}",
                     error=str(e),
                 ),
-                prefix="TEMPLATE",
+                prefix=self.prefix,
             )
             return self._generate_fallback_module(module_name, context, *components)
 
@@ -468,7 +480,7 @@ class AdvancedSVFeatureGenerator:
                 "Using fallback module generation for {module}",
                 module=module_name,
             ),
-            prefix="FALLBACK",
+            prefix=self.prefix,
         )
 
         header = generate_sv_header_comment(
@@ -515,7 +527,7 @@ endmodule
                 "Generating ports for {module_name}",
                 module_name=module_name,
             ),
-            prefix="PORTS",
+            prefix=self.prefix,
         )
 
         if module_name == "error_handler":
@@ -549,7 +561,7 @@ endmodule
                     "Unknown module type {module_name}, using default ports",
                     module_name=module_name,
                 ),
-                prefix="PORTS",
+                prefix=self.prefix,
             )
             return ""
 
@@ -563,19 +575,23 @@ endmodule
             "fatal_errors": list(self.config.error_handling.fatal_errors),
             "error_thresholds": self.config.error_handling.error_thresholds,
         }
-        return self.renderer.render_template("sv/error_recovery.sv.j2", context)
+        return self.renderer.render_template(
+            "sv/error_handling/error_recovery.sv.j2", context
+        )
 
     def _generate_error_logging_logic(self) -> str:
         """Generate error logging logic."""
         log_debug_safe(logger, "Generating error logging logic", prefix="ERROR_LOGIC")
 
         context = {"config": self.config.error_handling}
-        return self.renderer.render_template("sv/error_logging.sv.j2", context)
+        return self.renderer.render_template(
+            "sv/error_handling/error_logging.sv.j2", context
+        )
 
     def _generate_counter_logic(self) -> str:
         """Generate performance counter logic."""
         log_debug_safe(
-            logger, "Generating performance counter logic", prefix="PERF_LOGIC"
+            logger, "Generating performance counter logic", prefix=self.prefix
         )
 
         context = {"config": self.config.performance}
@@ -583,32 +599,32 @@ endmodule
 
     def _generate_sampling_logic(self) -> str:
         """Generate sampling logic."""
-        log_debug_safe(logger, "Generating sampling logic", prefix="PERF_LOGIC")
+        log_debug_safe(logger, "Generating sampling logic", prefix=self.prefix)
 
         context = {"config": self.config.performance}
         return self.renderer.render_template("sv/sampling_logic.sv.j2", context)
 
     def _generate_reporting_logic(self) -> str:
         """Generate reporting logic."""
-        log_debug_safe(logger, "Generating reporting logic", prefix="PERF_LOGIC")
+        log_debug_safe(logger, "Generating reporting logic", prefix=self.prefix)
 
         context = {"config": self.config.performance}
         return self.renderer.render_template("sv/reporting_logic.sv.j2", context)
 
     def _generate_state_machine(self) -> str:
         """Generate power state machine."""
-        log_debug_safe(logger, "Generating power state machine", prefix="POWER_LOGIC")
+        log_debug_safe(logger, "Generating power state machine", prefix=self.prefix)
 
         context = {"config": self.config.power_management}
         return self.renderer.render_template("sv/power_management.sv.j2", context)
 
     def _generate_clock_gating_logic(self) -> str:
         """Generate clock gating logic."""
-        log_debug_safe(logger, "Generating clock gating logic", prefix="POWER_LOGIC")
+        log_debug_safe(logger, "Generating clock gating logic", prefix=self.prefix)
 
         if not self.config.power_management.enable_clock_gating:
             log_debug_safe(
-                logger, "Clock gating disabled, skipping", prefix="POWER_LOGIC"
+                logger, "Clock gating disabled, skipping", prefix=self.prefix
             )
             return ""
 
@@ -617,9 +633,7 @@ endmodule
 
     def _generate_transition_logic(self) -> str:
         """Generate power transition logic."""
-        log_debug_safe(
-            logger, "Generating power transition logic", prefix="POWER_LOGIC"
-        )
+        log_debug_safe(logger, "Generating power transition logic", prefix=self.prefix)
 
         context = {"config": self.config.power_management}
         return self.renderer.render_template("sv/power_transitions.sv.j2", context)
