@@ -3,10 +3,8 @@
 from typing import Any, Dict, List, Set
 
 # Reuse shared validation constants instead of redefining overlapping values
-from src.utils import (
-    context_error_messages as CEM,
-    validation_constants as VC,
-)
+from src.utils import context_error_messages as CEM
+from src.utils import validation_constants as VC
 
 
 class SVConstants:
@@ -72,6 +70,7 @@ class SVConstants:
     DEFAULT_ENABLE_PERF_COUNTERS: bool = True
     DEFAULT_ENABLE_ERROR_DETECTION: bool = True
     DEFAULT_ENABLE_CUSTOM_CONFIG: bool = True
+    DEFAULT_OUT_OF_RANGE_SENTINEL: str = "DEADBEEF"
 
     # PCI config-space default words commonly referenced by generators
     DEFAULT_PCI_STATUS: int = 0x0010
@@ -210,12 +209,11 @@ class SVTemplates:
     DEVICE_SPECIFIC_PORTS: str = (
         "systemverilog/components/" "device_specific_ports.sv.j2"
     )
-    MAIN_ADVANCED_CONTROLLER: str = "systemverilog/advanced/advanced_controller.sv.j2"
-    CLOCK_CROSSING: str = "systemverilog/advanced/clock_crossing.sv.j2"
+    MAIN_ADVANCED_CONTROLLER: str = "sv/advanced_controller.sv.j2"
+    CLOCK_CROSSING: str = "sv/clock_crossing.sv.j2"
     BUILD_INTEGRATION: str = "python/build_integration.py.j2"
     PCILEECH_INTEGRATION: str = "python/pcileech_build_integration.py.j2"
-    PCILEECH_TLPS_BAR_CONTROLLER: str = "systemverilog/bar_controller.sv.j2"
-    ENHANCED_PCILEECH_TLPS_BAR_CONTROLLER: str = (
+    PCILEECH_TLPS_BAR_CONTROLLER: str = (
         "systemverilog/pcileech_tlps128_bar_controller.sv.j2"
     )
     PCILEECH_FIFO: str = "systemverilog/pcileech_fifo.sv.j2"
@@ -259,6 +257,14 @@ class SVValidation:
         "validation_failed": CEM.TEMPLATE_VALIDATION_FAILED,
         "missing_behavior_profile": CEM.MISSING_BEHAVIOR_PROFILE,
     }
+    NO_DONOR_DEVICE_IDS_ERROR: str = (
+        "Missing required device identifiers (vendor_id/device_id). "
+        "Cannot generate donor-unique firmware without these values."
+    )
+    NO_MSIX_HARDWARE_DATA_ERROR: str = (
+        "MSI-X table data must be read from actual hardware. "
+        "Cannot generate safe firmware without real MSI-X values."
+    )
 
 
 # Create singleton instances
