@@ -40,28 +40,45 @@ from src.string_utils import (
 )
 
 
-def format_hex_id(val: Union[int, str, None], width: int = 4) -> str:
+def format_hex_id(
+    val: Union[int, str, None], width: int = 4, permissive: bool = False
+) -> str:
     """
-    Format device ID values as hex strings with proper defaults for None values.
+    Format device ID values as hex strings with optional permissive defaults.
 
-    This function provides safe defaults for device identification values
-    to prevent template validation errors while maintaining backwards compatibility.
+    In strict mode (permissive=False), this enforces donor-uniqueness by requiring
+    explicit values and failing fast when None is encountered.
+
+    In permissive mode (permissive=True), legacy defaults are provided for
+    testing/development purposes.
 
     Args:
         val: Value to format (int, str, or None)
         width: Width of hex string (2, 4, or 6)
+        permissive: If True, use legacy defaults for None values
 
     Returns:
         Formatted hex string without 0x prefix
+
+    Raises:
+        ValueError: If val is None and permissive=False
     """
     if val is None:
-        # Return safe defaults instead of None to prevent template validation errors
-        if width == 2:
-            return "15"  # Default revision
-        elif width == 6:
-            return "020000"  # Default Ethernet class code
+        if permissive:
+            # Legacy defaults for testing/development
+            # These are NOT production values - use only in permissive mode
+            _PERMISSIVE_DEFAULTS = {
+                2: 0x15,  # revision_id default
+                4: 0x10EC,  # vendor_id default (Realtek)
+                6: 0x020000,  # class_code default (Ethernet controller)
+            }
+            val = _PERMISSIVE_DEFAULTS.get(width, 0x10EC)
         else:
-            return "10EC"  # Default Realtek vendor ID
+            raise ValueError(
+                f"Cannot format None value as hex ID (width={width}). "
+                "Donor-unique device identification values are required."
+            )
+
     # Coerce Enum values to their underlying integer before formatting
     if isinstance(val, Enum):
         # Prefer numeric value; if not available, fall back to string handling below
@@ -117,6 +134,13 @@ class BuildContext:
     build_jobs: int = 4
     build_timeout: int = 3600
 
+    # Optional donor-derived PCIe capability fields
+    # If provided, these will be mapped to Xilinx enum strings for use in templates
+    pcie_max_link_speed_code: Optional[int] = (
+        None  # 1=2.5, 2=5.0, 3=8.0, 4=16.0, 5=32.0
+    )
+    pcie_max_link_width: Optional[int] = None  # lane count: 1,2,4,8,16
+
     # PCILeech-specific parameters
     pcileech_src_dir: str = "src"
     pcileech_ip_dir: str = "ip"
@@ -134,13 +158,15 @@ class BuildContext:
         if not self.board_name:
             raise ValueError("board_name is required and cannot be empty")
 
-    def to_template_context(self, strict: bool = False) -> Dict[str, Any]:
+    # no-dd-sa
+    def to_template_context(self, strict: bool = True) -> Dict[str, Any]:
         """Convert build context to template context dictionary with all
         required variables.
 
         Args:
-            strict: If True, raises ValueError if any implicit defaults
-                are used
+            strict: If True (default), raises ValueError if any implicit defaults
+                are used, enforcing donor-uniqueness. If False, uses legacy
+                defaults for testing/development.
 
         Returns:
             Dictionary containing template context with metadata about
@@ -153,72 +179,80 @@ class BuildContext:
             "explicit_values": {},
         }
 
-        # Enhanced subsystem ID handling with proper defaults
-        subsys_vendor_id = getattr(self, "subsys_vendor_id", None) or self.vendor_id
-        subsys_device_id = getattr(self, "subsys_device_id", None) or self.device_id
+        # In strict mode, enforce donor-uniqueness: require all critical device IDs
+        if strict:
+            if self.vendor_id is None:
+                raise ValueError(
+                    "Strict mode enabled: Missing required vendor_id. Cannot generate "
+                    "donor-unique firmware without explicit vendor identification from "
+                    "donor device profiling."
+                )
+            if self.device_id is None:
+                raise ValueError(
+                    "Strict mode enabled: Missing required device_id. Cannot generate "
+                    "donor-unique firmware without explicit device identification from "
+                    "donor device profiling."
+                )
+            if self.revision_id is None:
+                raise ValueError(
+                    "Strict mode enabled: Missing required revision_id. Cannot generate "
+                    "donor-unique firmware without explicit revision from donor device "
+                    "profiling."
+                )
+            if self.class_code is None:
+                raise ValueError(
+                    "Strict mode enabled: Missing required class_code. Cannot generate "
+                    "donor-unique firmware without explicit class code from donor device "
+                    "profiling."
+                )
 
-        # Ensure all critical values have defaults to prevent None values
-        try:
-            # Prefer centralized fallbacks for vendor and class code
-            from src.device_clone.constants import VENDOR_ID_REALTEK
-            from src.templating.sv_constants import SVConstants
-
-            default_vendor = VENDOR_ID_REALTEK
-            default_class = SVConstants.DEFAULT_CLASS_CODE_INT
-        except Exception:
-            default_vendor = 0x10EC
-            default_class = 0x020000
-        # Preserve legacy default device_id expected by tests
-        default_device = 0x8168
-
-        vendor_id = self.vendor_id or default_vendor
-        device_id = self.device_id or default_device
-        revision_id = self.revision_id or 0x15  # Default revision
-        class_code = self.class_code or default_class  # Default to Ethernet controller
-
-        # Track which values were defaulted vs explicitly provided
-        if self.vendor_id is None:
-            context_metadata["defaults_used"]["vendor_id"] = default_vendor
+        # Apply values (with permissive defaults if not strict)
+        # Legacy defaults: vendor_id=0x10EC (Realtek), device_id=0x8168 (RTL8168)
+        # revision_id=0x15, class_code=0x020000 (Ethernet)
+        vendor_id = self.vendor_id
+        if vendor_id is None:
+            vendor_id = 0x10EC  # Legacy default
+            context_metadata["defaults_used"]["vendor_id"] = 0x10EC
         else:
-            context_metadata["explicit_values"]["vendor_id"] = self.vendor_id
+            context_metadata["explicit_values"]["vendor_id"] = vendor_id
 
-        if self.device_id is None:
-            context_metadata["defaults_used"]["device_id"] = default_device
+        device_id = self.device_id
+        if device_id is None:
+            device_id = 0x8168  # Legacy default
+            context_metadata["defaults_used"]["device_id"] = 0x8168
         else:
-            context_metadata["explicit_values"]["device_id"] = self.device_id
+            context_metadata["explicit_values"]["device_id"] = device_id
 
-        if self.revision_id is None:
+        revision_id = self.revision_id
+        if revision_id is None:
+            revision_id = 0x15  # Legacy default
             context_metadata["defaults_used"]["revision_id"] = 0x15
         else:
-            context_metadata["explicit_values"]["revision_id"] = self.revision_id
+            context_metadata["explicit_values"]["revision_id"] = revision_id
 
-        if self.class_code is None:
-            context_metadata["defaults_used"]["class_code"] = default_class
+        class_code = self.class_code
+        if class_code is None:
+            class_code = 0x020000  # Legacy default (Ethernet controller)
+            context_metadata["defaults_used"]["class_code"] = 0x020000
         else:
-            context_metadata["explicit_values"]["class_code"] = self.class_code
+            context_metadata["explicit_values"]["class_code"] = class_code
 
-        if getattr(self, "subsys_vendor_id", None) is None:
-            context_metadata["defaults_used"]["subsys_vendor_id"] = self.vendor_id
+        # Subsystem IDs: require explicit values or fallback to resolved main IDs
+        subsys_vendor_id = getattr(self, "subsys_vendor_id", None)
+        if subsys_vendor_id is None:
+            # Use resolved vendor_id (which has defaults applied if needed)
+            subsys_vendor_id = vendor_id
+            context_metadata["defaults_used"]["subsys_vendor_id"] = "vendor_id"
         else:
-            context_metadata["explicit_values"][
-                "subsys_vendor_id"
-            ] = self.subsys_vendor_id
+            context_metadata["explicit_values"]["subsys_vendor_id"] = subsys_vendor_id
 
-        if getattr(self, "subsys_device_id", None) is None:
-            context_metadata["defaults_used"]["subsys_device_id"] = self.device_id
+        subsys_device_id = getattr(self, "subsys_device_id", None)
+        if subsys_device_id is None:
+            # Use resolved device_id (which has defaults applied if needed)
+            subsys_device_id = device_id
+            context_metadata["defaults_used"]["subsys_device_id"] = "device_id"
         else:
-            context_metadata["explicit_values"][
-                "subsys_device_id"
-            ] = self.subsys_device_id
-
-        # In strict mode, reject any implicit defaults
-        if strict and context_metadata["defaults_used"]:
-            default_keys = list(context_metadata["defaults_used"].keys())
-            raise ValueError(
-                f"Strict mode enabled: Cannot use implicit defaults for "
-                f"{default_keys}. Please provide explicit values for these "
-                "fields."
-            )
+            context_metadata["explicit_values"]["subsys_device_id"] = subsys_device_id
 
         # Generate device signature for security compliance
         device_signature = safe_format(
@@ -302,6 +336,91 @@ class BuildContext:
         # Import TemplateObject for template compatibility
         from src.utils.unified_context import TemplateObject
 
+        # Derive and validate PCIe link speed/width enums from donor/IP context
+        def _map_speed(code: Optional[int], ip_type: str, strict: bool) -> str:
+            mapping = {
+                1: "2.5_GT/s",
+                2: "5.0_GT/s",
+                3: "8.0_GT/s",
+                4: "16.0_GT/s",
+                5: "32.0_GT/s",
+            }
+            if code is None:
+                if strict:
+                    raise TCLBuilderError(
+                        "Missing required PCIe link speed from donor device profiling. "
+                        "Cannot generate donor-unique firmware without speed capability data."
+                    )
+                # Non-strict mode: provide safe default for testing
+                # Track this default in context_metadata
+                context_metadata["defaults_used"]["pcie_max_link_speed_code"] = 2
+                return "5.0_GT/s"  # Gen2 - safe default for most 7-series boards
+            if code not in mapping:
+                raise TCLBuilderError(
+                    safe_format(
+                        "Invalid PCIe speed code {code} from donor device (valid: 1-5)",
+                        code=code,
+                    )
+                )
+            return mapping[code]
+
+        def _map_width(
+            width_val: Optional[int], default_lanes: int, strict: bool
+        ) -> str:
+            lanes = width_val or default_lanes
+            if lanes is None or lanes == 0:
+                if strict:
+                    raise TCLBuilderError(
+                        "Missing required PCIe link width from donor device profiling. "
+                        "Cannot generate donor-unique firmware without lane configuration data."
+                    )
+                # Non-strict mode: use max_lanes as fallback for testing
+                lanes = default_lanes if default_lanes > 0 else 1
+                context_metadata["defaults_used"]["pcie_max_link_width"] = lanes
+            if lanes < 1 or lanes > 16:
+                raise TCLBuilderError(
+                    safe_format(
+                        "Invalid PCIe link width {lanes} from donor device (valid: 1-16)",
+                        lanes=lanes,
+                    )
+                )
+            return f"X{lanes}"
+
+        def _validate_enums(ip_type: str, speed_enum: str, width_enum: str) -> None:
+            if ip_type == "pcie_7x":
+                allowed_speeds = {"2.5_GT/s", "5.0_GT/s"}
+                allowed_widths = {"X1", "X2", "X4", "X8"}
+            elif ip_type == "pcie_ultrascale":
+                allowed_speeds = {"2.5_GT/s", "5.0_GT/s", "8.0_GT/s", "16.0_GT/s"}
+                allowed_widths = {"X1", "X2", "X4", "X8", "X16"}
+            else:
+                return
+
+            if speed_enum not in allowed_speeds:
+                raise TCLBuilderError(
+                    safe_format(
+                        "Unsupported link speed {speed} for IP {ip}",
+                        speed=speed_enum,
+                        ip=ip_type,
+                    )
+                )
+            if width_enum not in allowed_widths:
+                raise TCLBuilderError(
+                    safe_format(
+                        "Unsupported link width {width} for IP {ip}",
+                        width=width_enum,
+                        ip=ip_type,
+                    )
+                )
+
+        derived_speed = _map_speed(
+            self.pcie_max_link_speed_code, self.pcie_ip_type, strict
+        )
+        derived_width = _map_width(self.pcie_max_link_width, self.max_lanes, strict)
+
+        # Validate derived enums against the IP type; fail fast on mismatch
+        _validate_enums(self.pcie_ip_type, derived_speed, derived_width)
+
         return {
             # REQUIRED VARIABLES - These are critical for template validation
             "device_signature": device_signature,
@@ -361,7 +480,7 @@ class BuildContext:
                     "coefficient_files": self.coefficient_file_list or [],
                 }
             ),
-            # Flat variables for backward compatibility (with safe defaults)
+            # Flat variables for backward compatibility (donor-derived values required)
             "board_name": self.board_name,
             "fpga_part": self.fpga_part,
             "pcie_ip_type": self.pcie_ip_type,
@@ -371,13 +490,10 @@ class BuildContext:
             "supports_msix": self.supports_msix,
             "synthesis_strategy": self.synthesis_strategy,
             "implementation_strategy": self.implementation_strategy,
-            "vendor_id": vendor_id,  # Use safe default
-            "device_id": device_id,  # Use safe default
-            "revision_id": revision_id,  # Use safe default
-            "class_code": class_code,  # Use safe default
-            "project_name": self.project_name,
-            "project_dir": self.project_dir,
-            "output_dir": self.output_dir,
+            "vendor_id": vendor_id,  # Required from donor device
+            "device_id": device_id,  # Required from donor device
+            "revision_id": revision_id,  # Required from donor device
+            "class_code": class_code,  # Required from donor device
             "project_name": self.project_name,
             "project_dir": self.project_dir,
             "output_dir": self.output_dir,
@@ -395,6 +511,9 @@ class BuildContext:
             "pcileech_ip_dir": self.pcileech_ip_dir,
             "batch_mode": self.batch_mode,
             "constraint_files": [],  # Add empty constraint files list
+            # Link configuration propagated to templates
+            "target_link_speed": derived_speed,
+            "target_link_width_enum": derived_width,
             # Context metadata for introspection and strict mode validation
             "context_metadata": context_metadata,
         }
@@ -573,6 +692,7 @@ class TCLBuilder:
         template_dir: Optional[Union[str, Path]] = None,
         output_dir: Optional[Union[str, Path]] = None,
         device_profile: Optional[str] = None,
+        prefix: str = "TCL_BLDER",
     ):
         """
         Initialize the TCL builder.
@@ -585,6 +705,7 @@ class TCLBuilder:
         self.logger = logging.getLogger(__name__)
         self.output_dir = Path(output_dir) if output_dir else Path(".")
         self.output_dir.mkdir(parents=True, exist_ok=True)
+        self.prefix = prefix
 
         # Initialize components
         self._init_template_renderer(template_dir)
@@ -647,6 +768,7 @@ class TCLBuilder:
             log_warning_safe(
                 self.logger,
                 safe_format("Device config module unavailable: {error}", error=e),
+                prefix=self.prefix,
             )
             self.device_config = None
 
@@ -681,7 +803,7 @@ class TCLBuilder:
             log_warning_safe(
                 self.logger,
                 safe_format("Using fallback constants: {error}", error=e),
-                prefix="CONSTANTS",
+                prefix=self.prefix,
             )
             fallback = self._create_fallback_constants()
             for attr_name in dir(fallback):
@@ -724,7 +846,7 @@ class TCLBuilder:
             log_warning_safe(
                 self.logger,
                 safe_format("Constraint manager unavailable: {error}", error=e),
-                prefix="CONSTRAINTS",
+                prefix=self.prefix,
             )
             self.constraint_manager = None
 
@@ -742,7 +864,20 @@ class TCLBuilder:
         return current
 
     def _select_board_interactively(self) -> str:
-        """Interactively select a board from available options."""
+        """Interactively select a board from available options.
+
+        Raises:
+            ValueError: If stdin is not a TTY or selection fails
+        """
+        # Check if stdin is a TTY (interactive terminal)
+        if not sys.stdin.isatty():
+            available_boards = list(self.BOARD_PARTS.keys())
+            raise ValueError(
+                "Board selection requires interactive terminal, but stdin is not a TTY. "
+                f"Please specify board explicitly using --board parameter. "
+                f"Available boards: {', '.join(available_boards)}"
+            )
+
         available_boards = list(self.BOARD_PARTS.keys())
         if not available_boards:
             raise ValueError("No board configurations available")
@@ -761,6 +896,12 @@ class TCLBuilder:
                 raise ValueError(f"Invalid selection: {selection}")
         except (ValueError, IndexError) as e:
             raise ValueError(f"Board selection failed: {e}") from e
+        except EOFError as e:
+            raise ValueError(
+                "Board selection failed: EOF while reading input. "
+                "This typically occurs in non-interactive environments (containers, CI/CD). "
+                f"Please specify board explicitly using --board parameter."
+            ) from e
 
     def create_build_context(
         self,
@@ -790,6 +931,10 @@ class TCLBuilder:
         Raises:
             ValueError: If required parameters are invalid
         """
+        # Normalize empty strings to None for consistent handling
+        board = board.strip() if board and isinstance(board, str) else board
+        board = None if board == "" else board
+
         # Determine board and FPGA part
         # Prefer non-interactive inference before prompting
         # 1) If board missing but fpga_part provided, reverse-map to a known board
@@ -886,6 +1031,30 @@ class TCLBuilder:
             self.device_config, "identification.subsystem_device_id"
         )
 
+        # Resolve final device identification values
+        final_vendor_id = vendor_id or config_vendor_id
+        final_device_id = device_id or config_device_id
+        final_revision_id = revision_id or config_revision_id
+        final_class_code = kwargs.get("class_code") or config_class_code or 0x020000
+
+        # Validate critical device identification values are present
+        # These are required for donor-unique firmware generation
+        missing_ids = []
+        if final_vendor_id is None:
+            missing_ids.append("vendor_id")
+        if final_device_id is None:
+            missing_ids.append("device_id")
+        if final_revision_id is None:
+            missing_ids.append("revision_id")
+
+        if missing_ids:
+            raise ValueError(
+                f"Missing required device identification values: {', '.join(missing_ids)}. "
+                "Cannot generate donor-unique firmware without complete device configuration. "
+                "Ensure device profiling/detection completed successfully and "
+                "all required parameters are provided."
+            )
+
         return BuildContext(
             board_name=board,
             fpga_part=fpga_part,
@@ -894,14 +1063,15 @@ class TCLBuilder:
             max_lanes=fpga_config.get("max_lanes", 1),
             supports_msi=fpga_config.get("supports_msi", False),
             supports_msix=fpga_config.get("supports_msix", False),
-            vendor_id=vendor_id or config_vendor_id,
-            device_id=device_id or config_device_id,
-            revision_id=revision_id or config_revision_id,
-            class_code=kwargs.get("class_code")
-            or config_class_code
-            or 0x020000,  # Default to Ethernet class
+            vendor_id=final_vendor_id,
+            device_id=final_device_id,
+            revision_id=final_revision_id,
+            class_code=final_class_code,
             subsys_vendor_id=subsys_vendor_id or config_subsys_vendor_id,
             subsys_device_id=subsys_device_id or config_subsys_device_id,
+            # Donor-derived PCIe link fields (optional)
+            pcie_max_link_speed_code=kwargs.get("pcie_max_link_speed_code"),
+            pcie_max_link_width=kwargs.get("pcie_max_link_width"),
             synthesis_strategy=kwargs.get(
                 "synthesis_strategy", self.SYNTHESIS_STRATEGY
             ),
@@ -945,10 +1115,13 @@ class TCLBuilder:
                     copied_files[0] if copied_files else None
                 )
             except XDCConstraintError as e:
-                log_error_safe(
-                    self.logger, safe_format("XDC constraint error: {error}", error=e)
+                log_warning_safe(
+                    self.logger,
+                    safe_format("XDC file handling failed: {error}", error=e),
+                    prefix=self.prefix,
                 )
                 raise
+                # Proceed without XDC files if copying fails
 
         # Add required variables for constraints template
         template_context.setdefault("sys_clk_freq_mhz", 100)  # Default to 100MHz
@@ -1244,7 +1417,9 @@ class TCLBuilder:
             return {filename: True for filename in tcl_contents.keys()}
         except Exception as e:
             log_error_safe(
-                self.logger, safe_format("Failed to write TCL files: {error}", error=e)
+                self.logger,
+                safe_format("Failed to write TCL files: {error}", error=e),
+                prefix=self.prefix,
             )
             # Return failure status for all files if batch write fails
             return {filename: False for filename in tcl_contents.keys()}
@@ -1421,6 +1596,8 @@ class TCLBuilder:
 
 
 # Backward compatibility aliases
+
+
 def create_tcl_builder(*args, **kwargs) -> TCLBuilder:
     """Factory function for creating TCL builder instances."""
     return TCLBuilder(*args, **kwargs)
