@@ -17,13 +17,6 @@ from pathlib import Path
 from typing import Any, Dict, Generic, List, Mapping, Optional, Set, TypeVar, Union
 
 from src.error_utils import extract_root_cause
-from src.utils.context_error_messages import (
-    MISSING_IDENTIFIERS,
-    STRICT_MODE_MISSING,
-    TEMPLATE_CONTEXT_VALIDATION_FAILED,
-)
-from src.utils.version_resolver import get_package_version
-
 from src.string_utils import (
     log_debug_safe,
     log_error_safe,
@@ -31,8 +24,15 @@ from src.string_utils import (
     log_warning_safe,
     safe_format,
 )
+from src.utils.context_error_messages import (
+    MISSING_IDENTIFIERS,
+    STRICT_MODE_MISSING,
+    TEMPLATE_CONTEXT_VALIDATION_FAILED,
+)
+from src.utils.version_resolver import get_package_version
 
 from .validation_constants import (
+    CORE_DEVICE_IDS,
     CRITICAL_TEMPLATE_CONTEXT_KEYS,
     DEFAULT_COUNTER_WIDTH,
     DEFAULT_PROCESS_VARIATION,
@@ -41,7 +41,6 @@ from .validation_constants import (
     DEVICE_CLASS_MAPPINGS,
     KNOWN_DEVICE_TYPES,
     POWER_TRANSITION_CYCLES,
-    CORE_DEVICE_IDS,
     SUBSYSTEM_ID_FIELDS,
 )
 
@@ -340,7 +339,7 @@ class SafeDefaults:
     voltage_variation = DEFAULT_VOLTAGE_VARIATION
 
 
-@dataclass
+@dataclass(slots=True)
 class UnifiedDeviceConfig:
     """Unified device configuration with all fields needed by templates."""
 
@@ -769,12 +768,20 @@ class UnifiedContextBuilder:
 
     def create_power_management_config(self, **kwargs) -> TemplateObject:
         """Create power management configuration for templates."""
+        from src.templating.sv_constants import SV_CONSTANTS
+
         config = dict(self.config.power_defaults)
 
         config.update(
             {
                 "enable_power_management": kwargs.get("enable_power_management", True),
                 "has_interface_signals": kwargs.get("has_interface_signals", False),
+                "pmcsr_bits": {
+                    "power_state_msb": SV_CONSTANTS.PMCSR_POWER_STATE_MSB,
+                    "power_state_lsb": SV_CONSTANTS.PMCSR_POWER_STATE_LSB,
+                    "pme_enable_bit": SV_CONSTANTS.PMCSR_PME_ENABLE_BIT,
+                    "pme_status_bit": SV_CONSTANTS.PMCSR_PME_STATUS_BIT,
+                },
             }
         )
 
@@ -920,6 +927,7 @@ class UnifiedContextBuilder:
         )
 
         # Build base context
+        from src.templating.sv_constants import SV_CONSTANTS
         from src.utils.validation_constants import SV_FILE_HEADER
 
         context = {
@@ -941,6 +949,13 @@ class UnifiedContextBuilder:
             "error_handling": error_handling_config,
             "error_config": error_handling_config,
             "performance_counters": perf_config,
+            # Add pmcsr_bits at top level for templates that access it directly
+            "pmcsr_bits": {
+                "power_state_msb": SV_CONSTANTS.PMCSR_POWER_STATE_MSB,
+                "power_state_lsb": SV_CONSTANTS.PMCSR_POWER_STATE_LSB,
+                "pme_enable_bit": SV_CONSTANTS.PMCSR_PME_ENABLE_BIT,
+                "pme_status_bit": SV_CONSTANTS.PMCSR_PME_STATUS_BIT,
+            },
             # Merge device signals
             **device_signals.to_dict(),
             # Template logic flags
@@ -1189,6 +1204,15 @@ class UnifiedContextBuilder:
             }
         )
 
+        # PCIe link configuration (safe defaults for testing)
+        # In production, these come from donor device profiling
+        context["target_link_speed"] = kwargs.get(
+            "target_link_speed", "5.0_GT/s"
+        )  # Gen2 default
+        context["target_link_width_enum"] = kwargs.get(
+            "target_link_width_enum", "X4"
+        )  # x4 lanes default
+
     def _add_compatibility_aliases(self, context: Dict[str, Any], **kwargs) -> None:
         """Add compatibility aliases for legacy templates."""
         # Update the main config object with error handling and performance
@@ -1338,8 +1362,47 @@ class UnifiedContextBuilder:
         context.setdefault(
             "integration_type", kwargs.get("integration_type", "default")
         )
-        context.setdefault("OVERLAY_ENTRIES", kwargs.get("OVERLAY_ENTRIES", []))
-        context.setdefault("OVERLAY_MAP", kwargs.get("OVERLAY_MAP", {}))
+        overlay_entries_value = kwargs.get(
+            "OVERLAY_ENTRIES", context.get("OVERLAY_ENTRIES", 0)
+        )
+
+        if isinstance(overlay_entries_value, (list, tuple, set, dict)):
+            normalized_overlay_entries = len(overlay_entries_value)
+        else:
+            try:
+                normalized_overlay_entries = int(overlay_entries_value)
+            except (TypeError, ValueError):
+                normalized_overlay_entries = 0
+
+        context.setdefault("OVERLAY_ENTRIES", normalized_overlay_entries)
+        context.setdefault("OVERLAY_MAP", kwargs.get("OVERLAY_MAP", []))
+
+        def _coerce_toggle(value: Any, default: int) -> int:
+            if value is None:
+                return default
+            if isinstance(value, str):
+                normalized = value.strip().lower()
+                if normalized in {"0", "false", "off", "no"}:
+                    return 0
+                if normalized in {"1", "true", "on", "yes"}:
+                    return 1
+            try:
+                return int(bool(value))
+            except Exception:
+                return default
+
+        sparse_default = _coerce_toggle(
+            kwargs.get("ENABLE_SPARSE_MAP"), int(normalized_overlay_entries > 0)
+        )
+        bit_types_default = _coerce_toggle(kwargs.get("ENABLE_BIT_TYPES"), 1)
+
+        context.setdefault("ENABLE_SPARSE_MAP", sparse_default)
+        context.setdefault("ENABLE_BIT_TYPES", bit_types_default)
+
+        hash_table_size = kwargs.get("HASH_TABLE_SIZE")
+        if hash_table_size is None:
+            hash_table_size = 16
+        context.setdefault("HASH_TABLE_SIZE", hash_table_size)
         context.setdefault("ROM_BAR_INDEX", kwargs.get("ROM_BAR_INDEX", 0))
         context.setdefault("FLASH_ADDR_OFFSET", kwargs.get("FLASH_ADDR_OFFSET", 0))
         context.setdefault("CONFIG_SHDW_HI", kwargs.get("CONFIG_SHDW_HI", 0xFFFF))
