@@ -7,17 +7,17 @@ and all methods for generating performance monitoring logic.
 """
 
 import logging
-from unittest.mock import Mock, patch, MagicMock
 from dataclasses import asdict
+from unittest.mock import MagicMock, Mock, patch
 
 import pytest
 
+from src.string_utils import log_info_safe, safe_format
 from src.templating.advanced_sv_perf import (
     DeviceType,
     PerformanceCounterConfig,
     PerformanceCounterGenerator,
 )
-from src.string_utils import log_info_safe, safe_format
 
 
 class TestDeviceType:
@@ -313,7 +313,7 @@ class TestPerformanceCounterGenerator:
 
         assert result == "transaction_counter_code"
         self.mock_renderer.render_template.assert_called_once_with(
-            "systemverilog/advanced/performance_counters.sv.j2",
+            "sv/performance_counters.sv.j2",
             generator._build_context_from_template_context({}),
         )
 
@@ -471,7 +471,7 @@ class TestPerformanceCounterGenerator:
 
         assert result == "complete_perf_code"
         self.mock_renderer.render_template.assert_called_once_with(
-            "systemverilog/advanced/performance_counters.sv.j2",
+            "sv/performance_counters.sv.j2",
             generator._build_context_from_template_context(template_context),
         )
 
