@@ -14,6 +14,7 @@ import pytest
 from src.device_clone.device_config import DeviceClass, DeviceType
 from src.exceptions import TemplateRenderError
 from src.templating.advanced_sv_features import ErrorHandlingConfig, PerformanceConfig
+from src.device_clone.overlay_utils import compute_sparse_hash_table_size
 from src.templating.advanced_sv_power import PowerManagementConfig
 from src.templating.systemverilog_generator import AdvancedSVGenerator
 
@@ -78,7 +79,7 @@ class TemplateContextBuilder:
     @staticmethod
     def create_minimal_context() -> Dict[str, Any]:
         """Create a minimal valid template context."""
-        return {
+        base_context = {
             "device_config": TemplateContextBuilder._create_device_config(),
             "device_signature": DEFAULT_DEVICE_SIGNATURE,
             "msix_config": TemplateContextBuilder._create_msix_config(),
@@ -90,8 +91,21 @@ class TemplateContextBuilder:
             "pcileech_config": TemplateContextBuilder._create_pcileech_config(),
             "generation_metadata": TemplateContextBuilder._create_generation_metadata(),
             "active_device_config": TemplateContextBuilder._create_active_device_config(),
+            "device_serial_number_int": 0x1234567890ABCDEF,
         }
 
+        base_context.update(
+            {
+                "OVERLAY_MAP": [],
+                "OVERLAY_ENTRIES": 0,
+                "ENABLE_SPARSE_MAP": 0,
+                "ENABLE_BIT_TYPES": 1,
+                "HASH_TABLE_SIZE": compute_sparse_hash_table_size(0),
+            }
+        )
+
+        return base_context
+
     @staticmethod
     def _create_device_config(
         vendor_id: str = DEFAULT_VENDOR_ID,
@@ -321,6 +335,7 @@ class TemplateContextBuilder:
         # Enhanced generation metadata
         context["generation_metadata"]["build_id"] = "test-build-123"
         context["generation_metadata"]["timestamp"] = "2024-01-01T12:00:00Z"
+        context["device_serial_number_int"] = 0xFEDCBA9876543210
 
         return context
 
